# -*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import re
import subprocess
import sys

from treepoem import _get_ghostscript_binary

GHOSTSCRIPT_VERSION = subprocess.check_output([
    _get_ghostscript_binary(), '--version'
]).decode('utf-8')
if not re.match(r'9\.\d\d', GHOSTSCRIPT_VERSION):
    print(
        "Ghostscript must be version 9.X, have {}".format(GHOSTSCRIPT_VERSION)
    )
    sys.exit(1)


def pytest_report_header(config):
    return "Ghostscript version: {}".format(GHOSTSCRIPT_VERSION)



# -*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import sys
from os import path

import pytest
from PIL import EpsImagePlugin, Image, ImageChops

import treepoem


@pytest.mark.parametrize('barcode_type,barcode_data', [
    ('qrcode', "This is qrcode barcode."),
    ('azteccode', "This is azteccode barcode."),
    ('pdf417', "This is pdf417 barcode."),
    ('interleaved2of5', "0123456789"),
    ('code128', "This is code128 barcode."),
    ('code39', "THIS IS CODE39 BARCODE."),
])
def test_barcode(barcode_type, barcode_data):
    actual = treepoem.generate_barcode(
        barcode_type,
        barcode_data,
        {}
    )

    fixture_path = "{dirname}/fixtures/{barcode_type}.png".format(
        dirname=path.dirname(__file__),
        barcode_type=barcode_type,
    )

    # Uncomment to rebuild fixtures:
    # actual.save(fixture_path)

    # Trying to prevent a `ResourceWarning`.
    # Bug: https://github.com/python-pillow/Pillow/issues/1144
    # Workaround: https://github.com/python-pillow/Pillow/issues/835
    with open(fixture_path, 'rb') as fixture:
        expected = Image.open(fixture)
        bbox = ImageChops.difference(actual, expected).getbbox()
        assert bbox is None

    actual.close()


@pytest.fixture
def pretend_windows():
    real_platform = sys.platform
    try:
        sys.platform = 'win32'
        yield
    finally:
        sys.platform = real_platform


@pytest.fixture
def pretend_have_windows_ghostscript():
    real_bin = EpsImagePlugin.gs_windows_binary
    try:
        EpsImagePlugin.gs_windows_binary = 'gswin32c'
        yield
    finally:
        EpsImagePlugin.gs_windows_binary = real_bin


def test_get_ghostscript_binary_windows(pretend_windows, pretend_have_windows_ghostscript):
    assert treepoem._get_ghostscript_binary() == 'gswin32c'


def test_get_ghostscript_binary_windows_missing(pretend_windows):
    with pytest.raises(treepoem.TreepoemError) as excinfo:
        treepoem._get_ghostscript_binary()
    assert 'Cannot determine path to ghostscript' in str(excinfo.value)


def test_unsupported_barcode_type():
    with pytest.raises(NotImplementedError) as excinfo:
        treepoem.generate_barcode('invalid-barcode-type', '', {})
    assert 'unsupported barcode type' in str(excinfo.value)



# -*- encoding:utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

# All supported barcode types, extracted from barcode.ps
barcode_types = {
    'auspost',
    'azteccode',
    'azteccodecompact',
    'aztecrune',
    'bc412',
    'channelcode',
    'codablockf',
    'code11',
    'code128',
    'code16k',
    'code2of5',
    'code32',
    'code39',
    'code39ext',
    'code49',
    'code93',
    'code93ext',
    'codeone',
    'coop2of5',
    'daft',
    'databarexpanded',
    'databarexpandedcomposite',
    'databarexpandedstacked',
    'databarexpandedstackedcomposite',
    'databarlimited',
    'databarlimitedcomposite',
    'databaromni',
    'databaromnicomposite',
    'databarstacked',
    'databarstackedcomposite',
    'databarstackedomni',
    'databarstackedomnicomposite',
    'databartruncated',
    'databartruncatedcomposite',
    'datalogic2of5',
    'datamatrix',
    'datamatrixrectangular',
    'dotcode',
    'ean13',
    'ean13composite',
    'ean14',
    'ean2',
    'ean5',
    'ean8',
    'ean8composite',
    'flattermarken',
    'gs1-128',
    'gs1-128composite',
    'gs1-cc',
    'gs1datamatrix',
    'gs1datamatrixrectangular',
    'gs1northamericancoupon',
    'gs1qrcode',
    'hanxin',
    'hibcazteccode',
    'hibccodablockf',
    'hibccode128',
    'hibccode39',
    'hibcdatamatrix',
    'hibcdatamatrixrectangular',
    'hibcmicropdf417',
    'hibcpdf417',
    'hibcqrcode',
    'iata2of5',
    'identcode',
    'industrial2of5',
    'interleaved2of5',
    'isbn',
    'ismn',
    'issn',
    'itf14',
    'japanpost',
    'kix',
    'leitcode',
    'matrix2of5',
    'maxicode',
    'micropdf417',
    'microqrcode',
    'msi',
    'onecode',
    'pdf417',
    'pdf417compact',
    'pharmacode',
    'pharmacode2',
    'planet',
    'plessey',
    'posicode',
    'postnet',
    'pzn',
    'qrcode',
    'rationalizedCodabar',
    'raw',
    'royalmail',
    'sscc18',
    'symbol',
    'telepen',
    'telepennumeric',
    'upca',
    'upcacomposite',
    'upce',
    'upcecomposite',
}



# -*- encoding:utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import codecs
import io
import os
import subprocess
import sys

from PIL import EpsImagePlugin

from .data import barcode_types

__all__ = ['generate_barcode', 'TreepoemError', 'barcode_types']
__version__ = '1.3.1'

BASE_DIR = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
BWIPP_PATH = os.path.join(BASE_DIR, 'postscriptbarcode', 'barcode.ps')

BASE_PS = """\
{bwipp}

/Helvetica findfont 10 scalefont setfont
gsave
2 2 scale
10 10 moveto

{code}
/uk.co.terryburton.bwipp findresource exec
grestore

showpage
"""

# Error handling from https://github.com/bwipp/postscriptbarcode/wiki/Developing-a-Frontend-to-BWIPP#use-bwipps-error-reporting
BBOX_TEMPLATE = """\
%!PS

errordict begin
/handleerror {{
  $error begin
  errorname dup length string cvs 0 6 getinterval (bwipp.) eq {{
    (%stderr) (w) file
    dup (\nBWIPP ERROR: ) writestring
    dup errorname dup length string cvs writestring
    dup ( ) writestring
    dup errorinfo dup length string cvs writestring
    dup (\n) writestring
    dup flushfile end quit
  }} if
  end //handleerror exec
}} bind def
end

""" + BASE_PS

EPS_TEMPLATE = """\
%!PS-Adobe-3.0 EPSF-3.0
{bbox}

""" + BASE_PS


class TreepoemError(RuntimeError):
    pass


# Inline the BWIPP code rather than using the run operator to execute
# it because the EpsImagePlugin runs Ghostscript with the SAFER flag,
# which disables file operations in the PS code.
def _read_file(file_path):
    with open(file_path) as f:
        return f.read()


BWIPP = _read_file(BWIPP_PATH)


def _get_bbox(code):
    full_code = BBOX_TEMPLATE.format(bwipp=BWIPP, code=code)
    ghostscript = _get_ghostscript_binary()
    gs_process = subprocess.Popen(
        [ghostscript,  '-sDEVICE=bbox', '-dBATCH', '-dSAFER', '-'],
        universal_newlines=True,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    _, err_output = gs_process.communicate(full_code)
    err_output = err_output.strip()
    # Unfortunately the error-handling in the postscript means that
    # returncode is 0 even if there was an error, but this gives
    # better error messages.
    if gs_process.returncode != 0 or 'BWIPP ERROR:' in err_output:
        if err_output.startswith('BWIPP ERROR: '):
            err_output = err_output.replace('BWIPP ERROR: ', '', 1)
        raise TreepoemError(err_output)
    return err_output


def _get_ghostscript_binary():
    binary = 'gs'

    if sys.platform.startswith('win'):
        binary = EpsImagePlugin.gs_windows_binary
        if not binary:
            raise TreepoemError(
                'Cannot determine path to ghostscript, is it installed?'
            )

    return binary


def _encode(data):
    return codecs.encode(data.encode('utf8'), 'hex_codec').decode('ascii')


def _format_options(options):
    items = []
    for name, value in options.items():
        if isinstance(value, bool):
            if value:
                items.append(name)
        else:
            items.append('{}={}'.format(name, value))
    return ' '.join(items)


def _format_code(barcode_type, data, options):
    return '<{data}> <{options}> <{barcode_type}> cvn'.format(
        data=_encode(data),
        options=_encode(_format_options(options)),
        barcode_type=_encode(barcode_type),
    )


def generate_barcode(barcode_type, data, options):
    if barcode_type not in barcode_types:
        raise NotImplementedError('unsupported barcode type {!r}'.format(barcode_type))
    code = _format_code(barcode_type, data, options)
    bbox_lines = _get_bbox(code)
    full_code = EPS_TEMPLATE.format(bbox=bbox_lines, bwipp=BWIPP, code=code)
    return EpsImagePlugin.EpsImageFile(io.BytesIO(full_code.encode('utf8')))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import codecs
import os
import re

from setuptools import find_packages, setup


def get_version(filename):
    with codecs.open(filename, 'r', 'utf-8') as fp:
        contents = fp.read()
    return re.search(r"__version__ = ['\"]([^'\"]+)['\"]", contents).group(1)


version = get_version(os.path.join('treepoem', '__init__.py'))

with codecs.open('README.rst', 'r', 'utf-8') as readme_file:
    readme = readme_file.read()

with codecs.open('HISTORY.rst', 'r', 'utf-8') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'Pillow',
]

setup(
    name='treepoem',
    version=version,
    description="Barcode rendering for Python 2 and 3 supporting QRcode, "
                "Aztec, PDF417, I25, Code128, Code39 and many more types.",
    long_description=readme + '\n\n' + history,
    author="Christian Muirhead",
    author_email='xtian@babbageclunk.com',
    maintainer="Julius Seporaitis",
    maintainer_email="julius@yplanapp.com",
    url='https://github.com/YPlan/treepoem',
    packages=find_packages(exclude=['tests', 'tests.*']),
    package_dir={
        'treepoem': 'treepoem',
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    keywords='barcode bwipp postscript ghostscript qr qrcode aztec azteccode pdf417 interleaved2of5 i25 code128 code39',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
